/************************************************************************
 *   IRC - Internet Relay Chat, src/digi.c
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include <sys/socket.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/stat.h>
#ifdef __hpux
#include "inet.h"
#endif
#if defined(PCS) || defined(AIX) || defined(DYNIXPTX) || defined(SVR3)
#include <time.h>
#endif
#ifdef  R_LINES
#include <signal.h>
#endif
#include "h.h"  
#include "msg.h"

extern char *klinefile;
extern char *configfile;
struct
{
        char    *name;
        char    *desc;
        int     (*function)(aClient*, int, char **);
} kline_funcs[] =
{
        { "ADD",  "Generate a K: line given userid and host.", kline_add },
        { "NICK", "Generate a K: line from a nick.",    kline_nick      },
        { "HELP", "List what commands work with KLINE", kline_help      },
        { NULL, NULL, (int(*)()) NULL }
};

/*
 * m_addcn
 * parv[0] = real hostname
 * parv[1] = password
 * parv[2] = network name
 * parv[3] = port
 */
int     m_addcn(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
FILE *fp;
int i, needrehash = 0;
char usrstr[300];
char cnvar[40][10];  
Reg1	aClient *acptr;
time_t	cur;
cur = time(NULL);

  	if (check_registered(sptr))
  		return -1;

	if (!IsAnOper(sptr))
	{
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

        if (parc < 4 || parc > 5)
	{
		sendto_one(sptr, "NOTICE %s :Help on \002ADDCN\002",parv[0]);
		sendto_one(sptr, 
			"NOTICE %s :Syntax: /quote ADDCN <ip> <password> <server name> [port]",
		parv[0]);
		return 0;
	}	

	if (strchr(parv[1], '*')||strchr(parv[2], '*')||strchr(parv[3], '*'))
	{
		sendto_one(sptr, "NOTICE %s :Dont use wildcards!",parv[0]);
		return 0;
	}

  	if (IsAnOper(sptr) || IsServer(cptr))
  	{
	sendto_one(sptr,"NOTICE %s :Adding C/N lines", parv[0]);
	sendto_ops("%s is adding C/N/H lines for: %s: %s",parv[0],parv[1],parv[3]);
        fp=fopen(configfile,"a");
         fprintf(fp,"# C/N Added by %s (%s@%s) on %s",
		 parv[0], sptr->user->username, sptr->sockhost, ctime(&cur)); 
         if (parc==5) 
                  {
                  sprintf(usrstr,"C:%s:%s:%s:%s:2\n",parv[1],parv[2],parv[3],parv[4]);
                  } 
       else if (parc==4) 
                  {
                  sprintf(usrstr,"C:%s:%s:%s::2\n",parv[1],parv[2],parv[3]);
                  } 
         fprintf(fp,"%s",usrstr); 
         fprintf(fp,"N:%s:%s:%s::2\n",parv[1],parv[2],parv[3]); 
	 fprintf(fp,"H:*::%s\n",parv[3]);
        fclose(fp);
	sendto_one(sptr, "NOTICE %s :%s", parv[0], usrstr);
	return rehash(cptr, sptr, 0);
	}
	return -1;
}

int     m_delcn(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
FILE	*in, *out;
char	buf[256];
char	temppath[256];
Reg1	aClient *acptr;
	sprintf(temppath, "%s.tmp",configfile);

        if (check_registered(sptr))
                return -1;

        if (!IsAnOper(sptr))
        {
                sendto_one(sptr,err_str(ERR_NOPRIVILEGES),me.name,parv[0]);
                return 0;
        }
	if (parc < 3 || parc > 3)
	{
	sendto_one(sptr, "NOTICE %s :Syntax /quote DELCN <ip/realhost> <servername>", 
		   parv[0]);
	return 0;
	}
        if (strchr(parv[1], '*')||strchr(parv[2], '*')) {
                sendto_one(sptr,"NOTICE %s :Dont use wildcards!",parv[0]);
                return 0; }
in=fopen(configfile,"r"); 
out=fopen(temppath,"w"); 
	while(fgets(buf,sizeof(buf), in)) {
	if (!strstr(buf,parv[2]) && !strstr(buf,parv[1])) 
	 fputs(buf,out);
	}
	fclose(in);
	fclose(out);
	rename(temppath,configfile);
	sendto_one(sptr, "NOTICE %s :%s has been removed", parv[0], parv[1]);
	sendto_ops("%s is removing C/N lines for %s: %s",parv[0],parv[1],parv[2]);
	return rehash(cptr, sptr, 0);
}

int addone_line(sptr, nick, host, name, reason)
aClient *sptr;
char *nick, *host, *name, *reason;
{
	aConfItem *tmp;
  	FILE 	*fp;
  	char 	*label, *option;
	char	confname[512];
 	time_t	cur;
  	
	label = (char *)MyMalloc(sizeof(char) * 256);
 	cur = time(NULL);
	if (!mycmp(host, "*.")||!mycmp(host, ".*")||!mycmp(host, "*.*")||
	    !mycmp(host, "*.edu")||!mycmp(host, "*.com")||!mycmp(host, "*.org")||
	    !mycmp(host, "*.net"))
	{
		sendto_one(sptr, "NOTICE %s :That would K-line everyone!", nick);
		return 0;
	}

	if (strchr(host, ':')||strchr(name, ':')||strchr(reason, ':'))
	{
                sendto_one(sptr,"NOTICE %s :Please dont use ':' in your K-Line info.",nick);
                return 0;
	}                                 		
  	(void)sprintf((char *)label,"# K:line Added by %s (%s@%s) on %s", 
		 nick, sptr->user->username, sptr->sockhost, ctime(&cur));

        for (tmp = conf; tmp; tmp = tmp->next)
	{
		if ((tmp->status == CONF_KILL)&&tmp->host&&tmp->name&&
                (!match(tmp->host, host)&&!match(tmp->name, name)))
			{
				sendto_one(sptr, "NOTICE %s :A Match for that K-Line exists. (K:%s:%s:%s)", nick, tmp->host, tmp->passwd, tmp->name);
				return 0;
			}
	}
	option = (char *)MyMalloc(sizeof(char)*1024);
	sprintf(option, "K:%s:%s:%s:0:-1\n", host, reason, name);
  	(void)alarm(3);

	sprintf(confname, "%s/%s",DPATH,klinefile);
  	fp = fopen(confname,"a");
  	(void)alarm(0);
	if (!fp) 
	{
  		sendto_one(sptr, "NOTICE %s :Error opening file %s",nick,confname);
	        MyFree(option);
	        MyFree(label);
  		return 0;
  	}
	fprintf(fp, "\n%s", label);
	fprintf(fp, "%s", option);
	fflush(fp);
  	fclose(fp);
	sendto_one(sptr, "NOTICE %s :Added K-line %s", nick, option);
	sendto_lops_butone(sptr, "%s added K-line %s", nick, option);
	MyFree(option);
  	MyFree(label);
  	return 1;
}

int kline_add(sptr, parc, parv)
aClient *sptr;
int parc;
char *parv[];
{
	char 	*host;
	char 	*userid = NULL;
	char 	*reason;
	int	argptr;
	
	if (parv[2]==NULL)
	{
		sendto_one(sptr, "NOTICE %s :No host given.", parv[0]);
		sendto_one(sptr, "NOTICE %s :Syntax: /quote KLINE ADD [userid] <host> [reason]", parv[0]);
		sendto_one(sptr, "NOTICE %s :If no userid is given * is assumed.", parv[0]);
		return -1;
	}
	argptr=2;

	if (!strchr(parv[argptr], '.')) userid = parv[argptr++];
	if (strchr(parv[argptr], '.')) host = parv[argptr++];
	else
	{
		sendto_one(sptr, "NOTICE %s :No host was found.", parv[0]);
		return -1;
	}

	if (parv[argptr]!=NULL)
	{
		reason = (char *)MyMalloc(sizeof(char)*512);
		strcpy(reason, parv[argptr++]);
		while(parv[argptr]) sprintf(reason, "%s_%s", reason, parv[argptr++]);
	}
	else reason = NULL;
	addone_line(sptr, parv[0], host,(userid)?userid:"*", (reason)?reason:"");
	if (reason!=NULL) MyFree(reason);
	return 1;
}	

int kline_nick(sptr, parc, parv)
aClient *sptr;
int parc;
char *parv[];
{
  	aClient	*acptr;
	char 	*name, *host;
	char 	*reason;
	int 	rptr = 0;

	if (parv[2]==NULL)
	{
		sendto_one(sptr, "NOTICE %s :You didn't specify a nick.", parv[0]);
		sendto_one(sptr, "NOTICE %s :Syntax: /quote KLINE NICK <nick> [reason]", parv[0]); 
		return 0;
	}
	if (!(acptr = find_client(parv[2], NULL)))
	{
		if (!(acptr = get_history(parv[1], (long)KILLCHASETIMELIMIT)))
		{
			sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, parv[0], parv[2]);	  	   		     
	        	return -1;
		}
	}
	if (!acptr->user) return -1; 

	name = acptr->user->username;
	host = acptr->sockhost;		
	
	/* Sanity checks */

	if (name == '\0' || host == '\0')
	{
		Debug((DEBUG_INFO, "KLINE: Error getting data."));
		sendto_one(sptr, "NOTICE %s :Error getting data.", parv[0]);
		return -1;
	}

	if (parv[3]!=NULL)
	{
		reason = (char *)MyMalloc(sizeof(char)*512);
		strcpy(reason, parv[3]);
		rptr=4;
		while(parv[rptr]) sprintf(reason, "%s_%s", reason, parv[rptr++]);
	}
	else reason = NULL;
	addone_line(sptr, parv[0], host, (name)?name:"*", (reason)?reason:"");
	if (reason!=NULL) MyFree(reason);
	return 1;
}

int kline_help(sptr, parc, parv)
aClient *sptr;
int parc;
char *parv[];
{
	int i;

	sendto_one(sptr, "NOTICE %s :Help on \002KLINE\002", parv[0]);	
	sendto_one(sptr, "NOTICE %s :Try /quote KLINE <option> for the syntax", parv[0]);
	for(i=0;kline_funcs[i].name;i++)
		sendto_one(sptr, "NOTICE %s :%s - %s", parv[0], kline_funcs[i].name, kline_funcs[i].desc);  
	return 0;
}

int     m_kline(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
 	int i, needrehash = 0;
  
  	if (check_registered_user(sptr))
  		return -1;

	if (parc < 2)
	{
		sendto_one(sptr,
		"NOTICE %s :Try /quote kline HELP for help with commands", parv[0]);
		return 0;
	}
	if ((parv[2]) && (parv[3]==NULL) && !strchr(parv[2], '.') &&
		!mycmp(parv[1], "add"))
	{
		sendto_one(sptr, "NOTICE %s :That will core the server!", parv[0]);
		return 0;
	}

  	if (IsAnOper(sptr) && MyConnect(sptr))
  	{
	        for(i=0;kline_funcs[i].name;i++)
	        {
	                if (!mycmp(parv[1], kline_funcs[i].name))
	                {
			        if (kline_funcs[i].function(sptr, parc, parv)==1) rehash(cptr, sptr, 2);
				return 1;
	                }
	        }
		sendto_one(sptr, "NOTICE %s :Try /quote kline HELP for help with commands", parv[0]);
  	}
	else sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);

  	return -1;
}

int     m_unkline(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
FILE    *in, *out;
char    buf[256];
char    temppath[256];
Reg1    aClient *acptr;
	sprintf(temppath, "%s.tmp",klinefile);
        if (check_registered(sptr))
                return -1;

        if (!IsVoper(sptr))
        {
                sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
                return 0;
        }
        if (parc < 2 || parc > 2)
        {
        sendto_one(sptr, "NOTICE %s :Syntax /quote UNKLINE <mask>",
                   parv[0]);
        return 0;
        }
        if (strchr(parv[1], '*'))
        {
                sendto_one(sptr, "NOTICE %s :Don't use wildcards!", parv[0]);
                return 0;
        }
	in=fopen(klinefile,"r");
	out=fopen(temppath,"w");
        while(fgets(buf, sizeof(buf), in)) {
        if(!strstr(buf,"K") && !strstr(buf, parv[1]))
	 fputs(buf, out);
        }
        fclose(in);
        fclose(out);
        rename(temppath,klinefile);
        sendto_one(sptr, "NOTICE %s :K:line %s has been removed", parv[0], parv[1]);
	sendto_ops("%s Removed K:Line: %s",parv[0],parv[1]);
        return rehash(cptr, sptr, 0);
}

int     m_cmdecho(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
   if (parv[1] != NULL){
      if (atoi(parv[1])) {
	SetCmdEcho(sptr);
         sendto_one(cptr,":%s NOTICE %s :Command echo enabled.",me.name,parv[0]);
      } else {
        ClearCmdEcho(sptr);
         sendto_one(sptr,":%s NOTICE %s :Command echo disabled.",me.name,parv[0]);
      }
   } else
	sendto_one(sptr,":%s NOTICE %s :Syntax: /quote cmdecho 1/0",me.name,parv[0]);
}

int     m_htm(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
        sendto_one(sptr, ":%s NOTICE %s :The incoming rate is %0.2f kb/s",
                me.name, parv[0], currentrate);
        return 0;
}
  
int     m_dog3freq(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
        int temp;

        if (!MyClient(sptr) || !IsAnOper(sptr))
        {
                sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
                return 0;
        }
        if (!parv[1] || !*parv[1])   
        {
                sendto_one(sptr, ":%s NOTICE %s :The current load check frequency is set at %i seconds",
                        me.name, parv[0], dog3loadcfreq);
                return 0;
        }
        temp = atoi(parv[1]);
        if (temp <= 0)
        {
                sendto_one(sptr, ":%s NOTICE %s :Hello???  Try a number > 0.",
                        me.name, parv[0]);
                return 0;
        }
        dog3loadcfreq = temp;
        sendto_ops("%s has changed the load check frequency to %i second(s).",
                parv[0], dog3loadcfreq);
        sendto_one(sptr, ":%s NOTICE %s :The load check frequency is now set to %i second(s)",
                me.name, parv[0], dog3loadcfreq);
        return 0;
}

int     m_dog3load(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
        int temp;
         
        if (!MyClient(sptr) || !IsAnOper(sptr))
        {
                sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
                return 0;
        }
        if (!parv[1] || !*parv[1])
        {
                sendto_one(sptr, ":%s NOTICE %s :The current load limit is set at %i kb",
                        me.name, parv[0], dog3loadrecv);
                return 0;
        }
        temp = atoi(parv[1]);
        if (temp < 0)
        {
                sendto_one(sptr, ":%s NOTICE %s :Hello???  Try a number >= 0.",
                        me.name, parv[0]);
                return 0;
        }
        dog3loadrecv = temp;
        sendto_ops("%s has changed the load limit to %i kb.",
                parv[0], dog3loadrecv);
        sendto_one(sptr, ":%s NOTICE %s :The load limit is now set to %i kb",
                me.name, parv[0], dog3loadrecv);
        return 0;
}

void addto_fdlist(fd,listp)
     int fd;
     fdlist *listp;
{
  register int index;  
  if ( (index = ++listp->last_entry) >= MAXCONNECTIONS) {
    /* list too big.. must exit */
    --listp->last_entry;
    exit(-1);
  }
  else
    listp->entry[index] = fd;
  return;  
}  
 
void delfrom_fdlist(fd,listp)
     int fd;
     fdlist *listp;
{
  register int i;
  for (i=listp->last_entry; i ; i--) {
    if (listp->entry[i]==fd) break; 
  }
  if (!i) return; /* could not find it! */
  /* swap with last_entry */
  if (i==listp->last_entry) {
    listp->entry[i] =0;
    listp->last_entry--;
    return;
  }
  else {
    listp->entry[i] = listp->entry[listp->last_entry];
    listp->entry[listp->last_entry] = 0;
    listp->last_entry--;
    return;
  }
}
 
void init_fdlist(listp)
     fdlist *listp;
{
  register int i;
  listp->last_entry=0;
  for (i=0; i<MAXCONNECTIONS+1; i++)
    listp->entry[i] = 0;
  return;
}

void read_motd(filename)
char *filename;
{
        int fd;
        register aMotd *temp, *last;
        struct stat sb;
        char    line[80];
        register char *tmp;

        while(motd)
        {
                temp = motd->next;  
                MyFree(motd);
                motd = temp;
        }


        if ((fd = open(MOTD, O_RDONLY)) == -1)
                return;
        (void)fstat(fd, &sb);
        motd_tm = localtime(&sb.st_mtime);
        (void)dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
        last = NULL;   
        while (dgets(fd, line, sizeof(line)-1) > 0)
        {
                if ((tmp = (char *)index(line,'\n')))
                        *tmp = '\0';
                if ((tmp = (char *)index(line,'\r')))
                        *tmp = '\0';
                temp = (aMotd *) MyMalloc(sizeof(aMotd));
                if (!temp)  
                        outofmemory();
                strcpy(temp->line, line);
                temp->next = NULL;
                if (!motd)
                        motd = temp;
                else
                        last->next = temp;
                last = temp;
        }
        (void)dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
        (void)close(fd);
}

void read_conf(filename)
char *filename;
{
        int fd; 
        register aDigi *temp, *last;
        struct stat sb;
        char    line[80]; 
        register char *tmp;  

        while(confstuff)
        {
                temp = confstuff->next;
                MyFree(confstuff);
                confstuff = temp;
        }
        if ((fd = open(configfile,O_RDONLY)) == -1)
                return;   
        (void)fstat(fd, &sb);
        digi_tm = localtime(&sb.st_mtime);
        (void)dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
        last = NULL;
        while (dgets(fd, line, sizeof(line)-1) > 0)
        {
                if ((tmp = (char *)index(line,'\n')))
                        *tmp = '\0';
                if ((tmp = (char *)index(line,'\r')))
                        *tmp = '\0';
                temp = (aDigi *) MyMalloc(sizeof(aDigi));
                if (!temp)
                        outofmemory();
                strcpy(temp->line, line);
                temp->next = NULL;
                if (!confstuff)
                        confstuff = temp;
                else
                        last->next = temp;
                last = temp;
        }
        (void)dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
        (void)close(fd);     
}

int     m_writeconf(aClient *cptr,aClient *sptr,int parc,char *parv[]) {
	FILE *fp;
        register aDigi *temp;
	static char *configfile_old="ircd.conf.old";
	time_t  cur;
	cur = time(NULL);
	if (check_registered(sptr))
                return 0;
        if (!IsAnOper(sptr))
        {
                sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
                return 0;
        }
        if (confstuff == NULL)
        {
                sendto_one(sptr,"NOTICE %s :Error: Configuration file is not in memory!",parv[0]);
                return 0;
        }
	rename(configfile,configfile_old);
	fp=fopen(configfile,"w");
        temp=confstuff;
        fprintf(fp,"# Re-Wrote configuration file: %s[%s@%s] on %s",
                 parv[0],sptr->user->username,sptr->sockhost,ctime(&cur));
        while(temp)
        {
                fprintf(fp,"%s\n",temp->line);
                temp=temp->next;
        }
	sendto_ops("%s[%s@%s] rewrote the configfile",parv[0],sptr->user->username,sptr->sockhost);
	fclose(fp);
        return 0;
}

char *slogans[] = {
	"Will you be my lamer?",
	"would you like fries with that?",
	"Paper or plastic?"
};
#define Slogan_Num 3
/*
** m_usage
**      parv[0] = sender prefix
**      parv[1] = usage topic
*/
int     m_usage(cptr, sptr, parc, parv)
aClient *cptr, *sptr;
int     parc;
char    *parv[];
{
	char *command;

                if (check_registered(sptr))
                        return 0;

	if (parv[1] != NULL) {	
        command = parv[1];
        if (!strncasecmp(command,"PRIVMSG",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
	sendto_one(cptr, ":%s NOTICE %s :Usage: PRIVMSG <nick/channel/server mask>[{,<nick/channel/server mask>}] :<message>", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :General message delivery command.  May be used to send private or public message.  Multiple recipients of either three types may be specified, separated by commas.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: PRIVMSG digi :Hello.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	PRIVMSG digi,SidE_ShoW,Lamerz,#valhalla :How's life?", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	PRIVMSG $*.com :Welcome, mere mortals.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	PRIVMSG #*netcom.com :Hello, folks!", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: NOTICE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"NOTICE",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick/channel/server mask>[{,<nick/channel/server mask>}] :<message>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Notice delivery command.\
	  May be used to send private or public message.  Multiple\
	recipients of either three types may be specified, separated by commas.\
	This MUST NOT trigger another event upon being received by\
		a client.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: NOTICE digi :Hello.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	NOTICE digi,SidE_ShoW,Lamerz,#valhalla :How's life?", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	NOTICE $*.com :Welcome, mere mortals.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	NOTICE #*netcom.com :Hello, folks!", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: PRIVMSG", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"ISON",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick>[{ <nick>}]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Nickname online query command.  Multiple nicknames may be stacked, separated by spaces.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: ISON digi", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :	ISON digi lamers NovaWolf", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: USERHOST", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"MODE",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick/channel> [<mode> <arguments>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Mode change command.  Valid recipients are the sender (user mode) and a channel (channel mode) on which the sender has channel operator privileges.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Mode control characters:", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   - - Unset mode.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   + - Set mode. (Default)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :User mode characters:", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   s - Receive server notices.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   w - Receive operator wallops.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   i - Invisible mode.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   d - Deaf mode. (Do not receive public messages)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   f - Operator only wallops. (Operators only)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   o - Operator mode. (Must be set via OPER, can only be unset via MODE)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   v - VOperator mode. (Must also be set via OPER, can only be unset via MODE)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel mode characters:", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   t - Only privileged users may change channel topic. (+v or +o users)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   n - No messages from users not on the channel may be sent to the channel.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   i - Invite-only channel.  Users wanting to JOIN must be invited with INVITE.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   m - Moderated channel.  Only privileged users may send public messages. (+v or +o users)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   s - Secret channel.  Channel will not show on LIST or in WHOIS channels list for outside users.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel mode characters requiring arguments:", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   l (<long int>)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :      - User limit.  Argument required with positive mode polarity.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   k <password>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :      - Channel key.  Users must include correct key with JOIN.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   b [<nick/user@host mask/@host mask/nick!user@host mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :      - Channel ban.  A nickname or mask may be specified; nicknames will be automatically expanded to a reasonable mask.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :      - Channel bans will be listed if mode argument is omitted.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :         - Examples:", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :            +b digi        - Mask automatically determined.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :            +b *!*@*net*ca - Full mask ban.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :            +b @*.??       - Masks all hostnames ending in two letters.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :            -b digi        - Automatically removes ban mask matching nickname, if found.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :            -b *!*@*net*ca - Removes full ban mask.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :             b             - List bans on channel.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   o <nick>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :      - Set/remove channel operator privileges.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :   v <nick>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :      - Set/remove channel voice privileges.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: MODE #darkstar o digi   - Give \"digi\" channel operator privileges.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          MODE #darkstar -o+nt da - Remove operator privileges from \"da\" and set +n and +t on channel.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          MODE digi +w-s         - If \"digi\" is the sender, enable wallops messages and disable server notices.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: UMODE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"WHO",3)) {
	sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick/channel/server/nick!user@host#server/ircname mask>[{,<nick/channel/server/mask>}]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :User information command, list format.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: WHO digi        - List information for \"digi\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          WHO S,digi      - List information for \"S\" and \"digi\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          WHO #darkstar   - List users on \"#darkstar\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          WHO foo.org     - List users on server \"foo.org\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          WHO *foo*       - List users containing \"foo\" in their nickname, username, hostname, servername, or ircname.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"WHOIS",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick> [<server/nick>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :User information command, detailed format.  The last argument may be specified to request a WHOIS from a remote server to obtain idle time/signon time/away time.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: WHOIS digi           - Show information for \"digi\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          WHOIS lamer digi     - Query the server of \"digi\" for information on \"lamer\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          WHOIS digi foo.org   - Query \"foo.org\" for information on digi.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   WHOIS digi digi      - Query the server \"digi\" is on for information about \"digi\"", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"WHOWAS",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Previous user information command, detailed format.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: WHOWAS digi          - List all occurances of \"digi\" in the WHOWAS database.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"NAMES",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<channel>[{,<channel>}]]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel user list command, short format.  Only nicknames are listed.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: NAMES           - List users on every (non-secret) channel.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          NAMES #darkstar - List users on \"#darkstar\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          NAMES #foo,#bar - List users on \"#foo\" and \"#bar\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"NICK",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Change nickname command.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Example: NICK digi       - Change nickname to \"digi\", if not already in use.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"JOIN",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :0/<channel>[{,0/<channel>}] [<key>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel join/create command.  Channel \"0\" will cause the sender to leave all channels.\
	A key argument is required for +k channels.  Valid channel names must start with either a '#' (global) or '&' (local to ser\ver).",
		me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: JOIN #foo        - Join channel \"#foo\", creating it if it does not already exist.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          JOIN &blah,#foo  - Join channels \"&blah\" and \"&foo\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          JOIN 0,#darkstar - Leave all channels and join \"#darkstar\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          JOIN 0           - Leave all channels.", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: PART", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"PART",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<channel>[{,<channel>}]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel part/leave command.  Use \"JOIN 0\" to part all channels.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: PART #lamers    - Leave \"#lamers\"", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          PART #foo,#bar  - Leave \"#foo\" and \"#bar\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: JOIN", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"QUIT",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[:<Disconnection comment>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Disconnect/signoff command.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: QUIT :Off to bed.  - Impossible case: user attempting sleep.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"TOPIC",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<channel> [:<new topic>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel topic view/change command.  Current topic is returned when no new topic argument is specified.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: TOPIC #boo         - Obtain current topic on \"boo\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          TOPIC #foo :stuff  - Change topic on \"#foo\" to \"stuff\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: MODE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"AWAY",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[:<away comment>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :User away/back command.  Away state is disabled when no argument is specified.  No message is automatically sent out to anyone other than the sender.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: AWAY                   - Unset away state.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          AWAY :Raiding fridge.  - Set away state with \"Raiding fridge.\" as the reason.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"LIST",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[[<channel/channel mask>] [[<minimum users>] [[<maximum users>]]]]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel list command.  All non-secret channels are listed when no arguments are specified.  Channel masks may be used.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: LIST             - List all non-secret channels.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          LIST #foo        - List channel \"#foo\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          LIST *bar*       - List all channels containing \"bar\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          LIST * 20        - List all channels with a minimum of twenty users.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          LIST *den* 5 15  - List all channels containing \"den\" with a minimum of five and a maximum of fifteen users.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: JOIN", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"LINKS",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server list command, cryptic format.  If a server mask is specified, the server and all servers further from the local server will be listed.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: LINKS          - List all servers on network.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          LINKS foo.org  - List \"foo.org\" and all servers further away.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: MAP", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"MAP",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server list command, graphical topography-based format.  A user count is included\
	for each server listed. If a server mask is specified, the server and all servers further from it will be listed.",
		me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: MAP          - Show all servers on network.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          MAP foo.org  - Show \"foo.org\" and all servers further away.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: LINKS", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"KICK",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<channel> <nick> [:<comment>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel user removal command.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: KICK #foo lamer  - Kick \"lamer\" from \"#foo\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          KICK #a b :c     - Kick \"b\" from \"#a\" with comment \"c\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: INVITE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"INVITE",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick> <channel>  - (Yes, the arguments are backwards...)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel invite user command. Must be used on +i channels.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Example: INVITE digi  #cave  - Invite \"digi\" to \"#cave\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: KICK", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"CHANINFO",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<channel>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Channel information command.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Example: CHANINFO #lamers    - Return channel information for \"#lamers\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"SILENCE",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[+<nick/nick!user@host mask>/-<nick/nick!user@host mask>/<nick>] [{...}]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :User silence command.  Stops server sending private messages/invites if the user responsible is on the SILENCE list.  With no arguments, SILENCE masks are listed.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: SILENCE +flooder   - Add \"flooder\" to the SILENCE list.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          SILENCE -flooder   - Remove mask matching \"flooder\", if possible.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          SILENCE +*!*@*.ca  - Add all hostnames ending in \".ca\" to the SILENCE list.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          SILENCE digi       - Return the SILENCE list of \"digi\".", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :          SILENCE            - List local SILENCE list.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"PING",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s ::<origin> <destination server>", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :PING command.  Used for server/network testing/benchmarking.", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: PONG", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"PONG",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s ::<data string>", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :PONG command.  Used for server/network testing/benchmarking.", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: PING", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"OPER",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<name> <password>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator command.  Used by server operators to set operator status.  Failed attempts are sent to all current local operators.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: OPER digi", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	OPER digi lame-o", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: MODE/UMODE(-o)", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"IRCNAME",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s ::<new ircname>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :IRCName change command.  The IRCName is the field displayed as the last argument in WHO.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Example: IRCNAME :John Doe", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: WHO", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"LANGUAGE",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<new language>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Numerics language change command.  If no argument is specified, available languages are listed.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Example: LANGUAGE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	LANGUAGE English", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"MOTD",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server MOTD request command.  If no argument is specified, the local MOTD is returned, if available.  Remote requests may be issued by including a server mask.",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Note: ircu+digi stores the MOTD in memory.  The configuration file must be reloaded if the MOTD is changed.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: MOTD", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	MOTD foo.org", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	MOTD foo*", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"VERSION",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server VERSION request command.  If no argument is specified, the local version is returned.  Remote requests may be issued by including a server mask.", me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Example: VERSION", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	  VERSION foo.org", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	  VERSION foo*", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"TIME",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server TIME request command.  If no argument is specified, the local time is returned.  Remote requests may be issued by including a server mask.", me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: TIME", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   TIME foo.org", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   TIME foo*", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"INFO",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server information request command.  If no argument is specified, the local information is returned.  Remote requests may be issued by including a server mask.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: INFO", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   INFO foo.org", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   INFO foo*", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: ADMIN", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"ADMIN",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server administration information request command.  If no argument is specified, the local information is returned.  Remote requests may be issued by including a server mask.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: ADMIN", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   ADMIN foo.org", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   ADMIN foo*", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: INFO", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"LUSERS",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<server mask>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :List users command.  LUSERS will return a list of local connections and remote\
	connections, including an invisible user and server count. Remote requests may be issued by including a server mask.",
		me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: LUSERS", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   LUSERS foo.org", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"STATS",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<character> [<server mask>]",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Server statistics command.  Uppercase requests match server configuration file lines.",
		me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: STATS O", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          STATS C foo.org", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"UMODE",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<mode>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :User mode command.  See MODE for mode details.", me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Example: UMODE +w", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: MODE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"WALLOPS",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s ::<message>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator wall command.  The message specified will be sent to all +w users.", me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: WALLOPS :Who just hacked my server?", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: SWALLOPS", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"SWALLOPS",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s ::<message>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator wall command.  The message specified will be sent to all operators umode +fonly.",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: SWALLOPS :I think I know who just hacked my server.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: WALLOPS", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"KILL",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<nick/channel/nick!user@host> [:<comment>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator KILL command.  Used by irc operators to disconnect users.  When using masks or channels, operators are omitted. USE MASKS CAREFULLY!", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: KILL digi        - Kill \"digi\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          KILL digi :FOO   - Kill \"digi\" with the comment \"FOO\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          KILL foobar     - Kill \"foobar\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          KILL #ircdsux    - Kill all users on \"#ircdsux\", excluding operators.",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          KILL *!*clone@*", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :                           - Kill everybody with \"clone\" in their username, excluding operators.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"REHASH",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :Operator rehash command.  Used by irc operators to reload the local server configuration file.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: RESTART DIE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"RESTART",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[:<comment>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator server restart command.  Used by irc operators to restart the server.", me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: DIE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"DIE",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[:<comment>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator server die/terminate command.  Used by irc operators to terminate the server.",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: RESTART", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"CONNECT",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<server mask> [<port>] [<to server>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator server connect command.  Remote requests may be issued by using a \"<to server>\" argument.  If no port is specified, the default from the configuration file is used.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: CONNECT *foo.org  - Connect to server in configuration file matching \"*foo.org\".", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          CONNECT a b       - Connect server \"a\" to server \"b\" on the default port.",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          CONNECT a 6667 b  - Connect server \"a\" to server \"b\" on port 6667.", me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          CONNECT *         - Connect to first server in configuration file.", me.name,parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: SQUIT", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"SQUIT",3) && IsOper(sptr)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<server mask>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator server quit command.  Masks matching multiple servers will only split locally connected servers.  A \"*\" for the server mask will SQUIT all local servers.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: SQUIT *           - Squit all servers locally connected.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          SQUIT foo.org     - Squit foo.org, disreguarding where it is connected.",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :          SQUIT *.ca        - Squit all servers with a suffix of *.ca", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: CONNECT", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"SETTIME",3) && IsOper(sptr)) {  
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :<time>/+<time>/-<time>", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Operator set server time command.  Used to set internal time delta.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: TIME", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"HELP",3)) {
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<command>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Command help.  If no argument is specified, all commands will be listed.  If a valid command is specified, help for the command will be returned.", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: HELP", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   HELP PRIVMSG", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: USAGE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else if (!strncasecmp(command,"USAGE",3)) { 
        sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
        sendto_one(cptr, ":%s NOTICE %s :[<command>]", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Command usage help.  If no argument is specified, usages of all commands will be listed.",me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :Examples: USAGE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :	   USAGE LANGUAGE", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :See also: PRIVMSG", me.name, parv[0]);
        sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"DIGI",3)) {
	sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
	sendto_one(cptr, ":%s NOTICE %s :The command digi will tell all the server running ircu+digi that they're\
		running it. All servers running ircu+digi must be connected for this to work properly.",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: VERSION", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"ADDCN",3) && IsOper(sptr)) {
	sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
	sendto_one(cptr, ":%s NOTICE %s :<real server host/ip> <connection password> <server name>",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :This command adds C/N/H lines for a server.",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :It writes to the ircd.conf, no wildcards allowed.",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :Examples: addcn 127.0.0.1 digi digi.com",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: DELCN",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"DELCN",3) && IsOper(sptr)) {
	sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
	sendto_one(cptr, ":%s NOTICE %s :<real host/ip> <server name>",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :This command removes C/N lines in the ircd.conf",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :No wildcards allowed!", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :Examples: DELCN 127.0.0.1 digi.com", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: ADDCN", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"KLINE",3) && IsOper(sptr)) {
	sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
	sendto_one(cptr, ":%s NOTICE %s :<type> [<mask>/<nick>] [comment]", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :Adds a K:Line to the K:line file", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :K:line types are: NICK, ADD", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :The NICK type adds a K:line based on a nick.", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :The ADD type adds a K:line matching the mask.", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :The spaces in the K:line comment are automatically turned into _'s",me.name,parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :Examples: KLINE NICK digi bye!", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :	   KLINE ADD *@digi.com bye!", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: UNKLINE", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
	else if (!strncasecmp(command,"UNKLINE",3) && IsOper(sptr)) {
	sendto_one(cptr, ":%s NOTICE %s :\002Help on %s\002", me.name, parv[0], command);
	sendto_one(cptr, ":%s NOTICE %s :<mask>", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :Removes a K:Line matching the mask. Wildcards NOT allowed.", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :Examples: UNKLINE *@digi.com", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :See also: KLINE", me.name, parv[0]);
	sendto_one(cptr, ":%s NOTICE %s :\002End help on %s\002", me.name, parv[0], command); }
        else {
        sendto_one(cptr, ":%s NOTICE %s :Invalid command: %s", me.name, parv[0], command); }
			} else {   
        int i;

        for (i = 0; msgtab[i].cmd; i++)
                sendto_one(sptr,":%s NOTICE %s :%s",
                           me.name, parv[0], msgtab[i].cmd);
        return 0;
    }
}
